/**
 * http://www.stat.ubc.ca/~bouchard/GEP/README-usage.html
 * 
 * @author Alexandre Bouchard
 * @author Ardavan Saeedi
 */
package gep.comparisons;
import java.io.File;
import java.util.Random;

import nuts.io.IO;
import nuts.math.RateMtxUtils;
import nuts.tui.Table;
import nuts.util.MathUtils;


public class TargetModel
{
  public final double [][] transitionRates;
  public final double [][] emiPrs;
  public TargetModel(double[][] transitionRates, double[][] emiPrs)
  {
    RateMtxUtils.checkRateMtx(transitionRates);
    for (double [] prs : emiPrs)
      MathUtils.checkIsProb(prs);
    this.transitionRates = transitionRates;
    this.emiPrs = emiPrs;
  }
  @Override
  public String toString()
  {
    return "RateMtx:\n" + Table.toString(transitionRates) + "\n"
     + "UnitTranMtx:\n" + Table.toString(RateMtxUtils.marginalTransitionMtx(transitionRates, 1.0)) + "\n" 
     + "EmiMtx:\n" + Table.toString(emiPrs);
  }
  public void save(File f)
  {
    f.mkdir();
    IO.writeToDisk(new File(f,"transitionRates.csv"), transitionRates);
    IO.writeToDisk(new File(f,"emissionPrs.csv"), emiPrs);
    IO.writeToDisk(new File(f,"nHiddenStates.txt"), "" + transitionRates.length);
    IO.writeToDisk(new File(f,"nObservedStates.txt"), "" + emiPrs[0].length);
  }
  
  public static TargetModel randomTargetType1(
      Random rand, 
      int hidSize, 
      int obsSize, 
      double p, // p as in ErdoRen(hidSize, p) random graph
      double small, // small rate
      double largeSmallDiff) // large rate  = small + largeSmallDiff
  {
    // emiPrs:
    double [][] emiPrs = randomDeterministic(rand, hidSize, obsSize);
    double [][] rateMtx = new double[hidSize][hidSize];
    for (int i = 0; i < hidSize; i++)
      for (int j = i + 1; j < hidSize; j++)
        rateMtx[i][j] = rateMtx[j][i] = small + (rand.nextDouble() < p ? largeSmallDiff : 0.0);
    RateMtxUtils.fillRateMatrixDiagonalEntries(rateMtx);
    return new TargetModel(rateMtx, emiPrs);
  }
  
  private static double[][] randomDeterministic(Random rand, int hidSize, int obsSize)
  {
    double [][] result = new double[hidSize][obsSize];
    for (int i = 0; i < hidSize; i++)
      result[i][rand.nextInt(obsSize)] = 1.0;
    return result;
  }
}