/**
 * http://www.stat.ubc.ca/~bouchard/GEP/README-usage.html
 * 
 * @author Alexandre Bouchard
 * @author Ardavan Saeedi
 */
package gep.data;
import java.io.*;
import java.util.*;
import java.util.regex.Pattern;

import pepper.Encodings;
import pty.RootedTree;
import pty.io.Dataset;
import pty.learn.DiscreteBP;
import pty.smc.models.CTMC;

import fig.basic.LogInfo;
import fig.basic.Option;
import gep.timeseries.Measurements;
import goblin.Taxon;
import ma.MSAParser;
import ma.MSAPoset;
import ma.RateMatrixLoader;
import ma.SequenceType;
import nuts.io.IO;
import nuts.math.GMFct;
import nuts.util.CollUtils.*;
import nuts.util.Arbre;
import nuts.util.CollUtils;
import nuts.util.Counter;
import nuts.util.Indexer;
import static nuts.util.CollUtils.*;
import static nuts.io.IO.*;
import static nuts.util.MathUtils.*;


public class LoadFlatDataFormat implements DataSource
{
  @Option public String flatDataFile = "";
  
  private List<Measurements> _data = null;
  private int pointer = 0;
  
  public static final Pattern IGNORE_LINE_MATCH = Pattern.compile("^([#].*)?\\s*$");
  
  private void ensureLoaded(Indexer<String> indexer)
  {
    if (_data != null) 
      return;
    _data = CollUtils.list();
    _possibleObs = CollUtils.set();
    
    _read(false, indexer);
  }
  
  private void _read(boolean justPeakingValue, Indexer<String> indexer)
  {
    int index = 0;
    List<Double> times = list();
    List<Integer> values = list();
    for (String line : IO.i(flatDataFile))
      if (!IGNORE_LINE_MATCH.matcher(line).matches())
      {
        String [] fields = line.split("\\s+");
        if (index++ % 2 == 0)
        {
          // process times
          if (!justPeakingValue)
            for (String field : fields)
              times.add(Double.parseDouble(field));
        }
        else
        {
          // process observed states
          for (String field : fields)
          {
            if (justPeakingValue)
              _possibleObs.add(field);
            else
              values.add(indexer.o2i(field)); 
          }
          if (!justPeakingValue)
          {
            Measurements currentM = new Measurements(times, values);
            _data.add(currentM);
            times = list();
            values = list();
          }
        }
      }
    
    if (times.size() != 0)
      throw new RuntimeException();
  }

  @Override
  public Measurements next(Indexer<String> indexer)
  {
    ensureLoaded(indexer);
    if (pointer++ < _data.size())
      return _data.get(pointer-1);
    else
      return null;
  }
  
  private Set<String> _possibleObs;
  
  @Override
  public Set<String> possibleObservations()
  {
    if (_possibleObs == null)
    {
      _possibleObs = CollUtils.set();
      _read(true, null);
    }
    return _possibleObs;
  }

  
}
