/**
 * http://www.stat.ubc.ca/~bouchard/GEP/README-usage.html
 * 
 * @author Alexandre Bouchard
 * @author Ardavan Saeedi
 */
package gep.model;
import java.util.Random;

import nuts.util.Counter;


public class SplitBaseMeasure implements BaseMeasure<SplitContext,SplitContext>
{
  private final double alpha0;
  private final int nSymbols;
  public SplitBaseMeasure(double alpha0, int nSymbols)
  {
    this.alpha0 = alpha0;
    this.nSymbols = nSymbols;
  }
  @SuppressWarnings("unchecked")
  @Override
  public SplitContext sampleState(Random rand,
      SufficientStatistics<SplitContext, SplitContext> readWriteSuffStat,
      SufficientStatistics<SplitContext, SplitContext> readOnlySuffStats)
  {
    final double unif = rand.nextDouble();
    @SuppressWarnings("rawtypes")
    Counter ssRW = readWriteSuffStat.getBaseMeasureStats(),
            ssR  = readOnlySuffStats.getBaseMeasureStats();
    final double nObs = ssRW.totalCount() + ssR.totalCount();
    Integer result = null;
    if (rand.nextDouble() < nObs / (nObs + alpha0))
      // use cache
      result = (Integer) Predictives.sampleCache(rand, ssRW, ssR);
      // sample new
    else
      result = rand.nextInt(nSymbols);
    ssRW.incrementCount(result, 1.0);
    return new SplitContext(unif, result);
  }
  
}