/**
 * http://www.stat.ubc.ca/~bouchard/GEP/README-usage.html
 * 
 * @author Alexandre Bouchard
 * @author Ardavan Saeedi
 */
package gep.model;
import static nuts.util.CollUtils.list;

import java.util.Collections;
import java.util.List;
import java.util.Random;

import nuts.util.Counter;
import fig.prob.SampleUtils;

/**
 * Hyper params for the state-split model
 *
 */
public class SplitHyperParams implements HyperParamProvider<SplitContext>
{
  private final List<Double> alphaLevel1 = list(); // these may be resampled easily
  private final double alphaLevel2, alphaLevel3;   // these are fixed for now
  private final double gamma0;                     // these are fixed for now
  public SplitHyperParams(double alphaLevel1, double alphaLevel2, double alphaLevel3, int nSym, double gamma0)
  {
    for (int i = 0; i < nSym; i++)
      this.alphaLevel1.add(alphaLevel1);
    this.alphaLevel2 = alphaLevel2;
    this.alphaLevel3 = alphaLevel3;
    this.gamma0 = gamma0;
  }
  public List<Double> getGammas() { return Collections.unmodifiableList(alphaLevel1); }
  //get the concentration parameter for \bar \mu_context
  @Override
  public double getHyperParam(SplitContext context)
  {
    int level = context.level();
         if (level == 0) return alphaLevel1.get(context.getX()); // getting the normalization of the parent level GMP
    else if (level == 1) return alphaLevel2;
    else if (level == 2) return alphaLevel3;
    else
      throw new RuntimeException();
  }
  // shape parameter for resampling one of alphaLevel1
  private double getA(int x, SufficientStatistics<SplitContext,SplitContext> stats)
  {
    SplitContext xCtxt = new SplitContext(x);
    return alphaLevel2 + stats.getStateTransitionStats().getCounter(xCtxt).totalCount();
  }
  // rate parameter for resampling one of alphaLevel1
  private double getB(int x, SufficientStatistics<SplitContext,SplitContext> stats, double beta0)
  {
    double sum = gamma0;
    Counter<SplitContext> jumpStats = stats.getWaitingTimeStats();
    for (SplitContext ctxt : jumpStats.keySet())
      if (ctxt.getX() == x)
        sum += Math.log(beta0 + jumpStats.getCount(ctxt)) - Math.log(beta0);
    if (!(sum > 0))
      throw new RuntimeException("getB() returned " + sum + "\nbeta0=" + beta0 + ",jumpStats=" + jumpStats);
    return sum;
  }
  private void sampleGamma(Predictives<SplitContext,SplitContext> predictiveDistributions, Random rand, int charIndex)
  {
    // get suff stats
    final SufficientStatistics<SplitContext,SplitContext> stats = predictiveDistributions.readOnlySuffStats;
    final double beta0 = predictiveDistributions.beta0;
    // get params
    final double a = ((SplitHyperParams)predictiveDistributions.hyperParams).getA(charIndex, stats);
    final double b = ((SplitHyperParams)predictiveDistributions.hyperParams).getB(charIndex, stats, beta0);
    
    double newValue = SampleUtils.sampleGamma(rand, a, b);
    if (!(newValue > 0))
      throw new RuntimeException("sampleGamma() returned " + newValue + ", a=" + a + "," + "b=" + b);
//    LogInfo.logs("Gamma move " + charIndex + ":" + alphaLevel1.get(charIndex) + "->" + newValue);
    alphaLevel1.set(charIndex, newValue);
  }
  public void sample(Predictives<SplitContext,SplitContext> predictiveDistributions, Random rand)
  {
    for (int charIdx = 0; charIdx < alphaLevel1.size(); charIdx++)
      sampleGamma(predictiveDistributions, rand, charIdx);
  }
}