/**
 * http://www.stat.ubc.ca/~bouchard/GEP/README-usage.html
 * 
 * @author Alexandre Bouchard
 * @author Ardavan Saeedi
 */
package gep.model;
import java.util.Random;
import java.util.Set;

import nuts.util.CollUtils;
import nuts.util.Counter;
import nuts.util.CounterMap;
import nuts.util.MathUtils;


public class SufficientStatisticsImpl<C,S> implements SufficientStatistics<C,S>
{
  @SuppressWarnings("unchecked")
  private Counter bmStats = new Counter();
  private CounterMap<C,S> stateTransStats = new CounterMap<C,S>();
  private Counter<C> waitingTimeStats = new Counter<C>();
  @SuppressWarnings("unchecked")
  @Override
  public Counter getBaseMeasureStats()
  {
    return bmStats;
  }
  @Override
  public CounterMap<C,S> getStateTransitionStats()
  {
    return stateTransStats;
  }
  @Override
  public Counter<C> getWaitingTimeStats()
  {
    return waitingTimeStats;
  }
  public SufficientStatisticsImpl() {}
  @SuppressWarnings("unchecked")
  public SufficientStatisticsImpl(SufficientStatistics<C,S> model)
  {
    this.bmStats.incrementAll(model.getBaseMeasureStats());
    this.waitingTimeStats.incrementAll(model.getWaitingTimeStats());
    this.stateTransStats.incrementAll(model.getStateTransitionStats());
  }
  @Override
  public String toString()
  {
    return "StateTrans:" + stateTransStats.toString() + "\nWaitTimes:" + waitingTimeStats.toString() + "\nBM:" + bmStats.toString();
  }
  
  public static int nHiddenStates(SufficientStatistics<SplitContext,SplitContext> ss)
  {
    return nHiddenStates(ss.getStateTransitionStats());
  }
  public static int nHiddenStates(CounterMap<SplitContext,SplitContext> stateTransStats)
  {
    Set<Double> set = CollUtils.set();
    for (SplitContext sc1 : stateTransStats.keySet())
    {
      Counter<SplitContext> c  = stateTransStats.getCounter(sc1);
      if (sc1.level() == 0 && c.totalCount() > 0)
        set.add(sc1.getU());
      for (SplitContext sc2 : c.keySet())
        if (stateTransStats.getCount(sc1,sc2) > 0)
          if (sc2.level() == 0)
            set.add(sc2.getU());
    }
    return set.size();
  }
  
  //dest += src
  @SuppressWarnings("unchecked")
  public static <C,S> void plusEqual(SufficientStatistics<C,S> dest, SufficientStatistics<C,S> src)
  {
    dest.getBaseMeasureStats().incrementAll(src.getBaseMeasureStats());
    dest.getWaitingTimeStats().incrementAll(src.getWaitingTimeStats());
    dest.getStateTransitionStats().incrementAll(src.getStateTransitionStats());
  }
  // dest -= src; also checks the new values are non-negative
  @SuppressWarnings("unchecked")
  public static <C,S> void minusEqualCheckNonNeg(SufficientStatistics<C,S> dest, SufficientStatistics<C,S> src)
  {
    {
      CounterMap<C,S> destcm = dest.getStateTransitionStats(), 
                      srccm  = src.getStateTransitionStats();
      for (C key1 : srccm.keySet())
        for (S key2 : srccm.getCounter(key1).keySet())
        {
          double value = destcm.getCount(key1, key2) - srccm.getCount(key1, key2);
          if (MathUtils.close(0.0, value))
            value = 0.0;
          if (value < 0)
            throw new RuntimeException();
          destcm.setCount(key1, key2, value);
        }
    }
    {
      Counter destc = dest.getBaseMeasureStats(), 
              srcc  = src.getBaseMeasureStats();
      for (Object key : srcc.keySet())
      {
        double value = destc.getCount(key) - srcc.getCount(key);
        if (MathUtils.close(0.0, value))
          value = 0.0;
        if (value < 0)
          throw new RuntimeException();
        destc.setCount(key, value);
      }
    }
    {
      Counter destc = dest.getWaitingTimeStats(),
              srcc  = src.getWaitingTimeStats();
      for (Object key : srcc.keySet())
      {
        double value = destc.getCount(key) - srcc.getCount(key);
        if (MathUtils.close(0.0, value))
          value = 0.0;
        if (value < 0)
          throw new RuntimeException();
        destc.setCount(key, value);
      }
    }
  }
  @SuppressWarnings("unchecked")
  public static void main(String [] args)
  {
    Random rand = new Random(1);
    SufficientStatistics<Integer,Integer> ss2 = new SufficientStatisticsImpl<Integer,Integer>();
    for (int i = 0; i < 10; i++)
      ss2.getBaseMeasureStats().incrementCount(rand.nextInt(), rand.nextDouble());
    for (int i = 0; i < 10; i++)
      ss2.getWaitingTimeStats().incrementCount(rand.nextInt(), rand.nextDouble());
    for (int i = 0; i < 10; i++)
      for (int i2 = 0; i2 < 10; i2++)
        ss2.getStateTransitionStats().incrementCount(i,i2,rand.nextDouble());
    
    
    SufficientStatistics<Integer,Integer> ss = new SufficientStatisticsImpl<Integer,Integer>();
    for (int i = 0; i < 10; i++)
      ss.getBaseMeasureStats().incrementCount(rand.nextInt(), rand.nextDouble());
    for (int i = 0; i < 10; i++)
      ss.getWaitingTimeStats().incrementCount(rand.nextInt(), rand.nextDouble());
    for (int i = 0; i < 10; i++)
      for (int i2 = 0; i2 < 10; i2++)
        ss.getStateTransitionStats().incrementCount(i,i2,rand.nextDouble());
    
    System.out.println(ss2);
    plusEqual(ss2, ss);
    System.out.println(ss2);
    minusEqualCheckNonNeg(ss2, ss);
    
    System.out.println(ss2);
  }
}