/**
 * http://www.stat.ubc.ca/~bouchard/GEP/README-usage.html
 * 
 * @author Alexandre Bouchard
 * @author Ardavan Saeedi
 */
package gep.pmcmc;
import gep.model.SufficientStatistics;
import gep.timeseries.Event;

import java.util.Collections;
import java.util.List;


public class PartialHiddenState<S>
{
  private final SufficientStatistics<S,S> currentStats;
  private final List<Event<S>> events;
  private final int nObservationAskedToCover; // the actual number of observations covered might be higher
  
  public PartialHiddenState(SufficientStatistics<S, S> currentStats,
      List<Event<S>> events, int nObservationAskedToCover)
  {
    this.currentStats = currentStats;
    this.events = events;
    this.nObservationAskedToCover = nObservationAskedToCover;
  }
  public List<Event<S>> eventsList()
  {
    return Collections.unmodifiableList(events);
  }
  public int nObservationAskedToCover()
  {
    return nObservationAskedToCover;
  }
  public double eventsLength()
  {
    if (events.isEmpty()) return 0.0;
    return events.get(events.size()-1).right();
  }
  public SufficientStatistics<S,S> suffStats()
  {
    return currentStats;
  }
}
