/**
 * http://www.stat.ubc.ca/~bouchard/GEP/README-usage.html
 * 
 * @author Alexandre Bouchard
 * @author Ardavan Saeedi
 */
package gep.util;
import java.io.File;
import java.io.PrintWriter;
import java.util.Map;

import nuts.io.CSV;
import nuts.util.CollUtils;
import fig.basic.IOUtils;
import fig.basic.LogInfo;
import fig.exec.Execution;


public class OutputManager
  {
    private Map<String, PrintWriter> _writers = CollUtils.map();
    private File _resultsFolder = null;
    private final int flushInterval = 5;
    private int current = 0;
    private PrintWriter getWriter(String key)
    {
      if ((++current % flushInterval) == 0) flush();
      PrintWriter result = _writers.get(key);
      if (result != null) return result;
      result = IOUtils.openOutHard(new File(getResultsFolder(), key + ".csv"));
      _writers.put(key, result);
      return result;
    }
    private boolean hasWriter(String key) { return _writers.containsKey(key); }
    private File getResultsFolder()
    {
      if (_resultsFolder != null) return _resultsFolder;
      _resultsFolder = new File(Execution.getFile("results"));
      _resultsFolder.mkdir();
      return _resultsFolder;
    }
    public void printWrite(String masterKey, Object... keyValues)
    {
      write(masterKey, keyValues);
      StringBuilder toPrint = new StringBuilder();
      toPrint.append(masterKey + ": ");
      final int len = keyValues.length/2;
      for (int i = 0; i < len; i++)
        toPrint.append("" + keyValues[i*2] + "=" + keyValues[i*2 + 1] + (i == len - 1 ? "" : ", "));
      LogInfo.logsForce(toPrint);
    }
    public void write(String masterKey, Object... keyValues)
    {
      if ((keyValues.length % 2) != 0) 
      {
        LogInfo.warning("Malformed OutputManager.print()");
        return;
      }
      final boolean needPrintHeader = !hasWriter(masterKey);
      final int len = keyValues.length/2;
      Object [] heads = (needPrintHeader ? new Object[len] : null);
      Object [] values= new Object[len];
      for (int i = 0; i < len; i++)
      {
        Object key = keyValues[i*2],
               value=keyValues[i*2+1];
        if (needPrintHeader)
          heads[i] = key;
        values[i] = value; 
      }
      if (needPrintHeader)
        printHeader(masterKey, heads);
      printBody(masterKey, values);
    }
    private void printHeader(String key, Object... objects)
    {
      getWriter(key).println(CSV.header(objects));
    }
    private void printBody(String key, Object... objects)
    {
      getWriter(key).println(CSV.body(objects));
    }
//    public void printLogBody(String key, int fieldToStdout, Object... objects)
//    {
//      try { LogInfo.logsForce("" + key + ": " + objects[fieldToStdout]); }
//      catch (Exception e) { e.printStackTrace(); }
//      printBody(key, objects);
//    }
    public void close()
    {
      for (PrintWriter out : _writers.values())
        out.close();
    }
    public void flush()
    {
      for (PrintWriter out : _writers.values())
        out.flush();
    }
  }