/**
 * http://www.stat.ubc.ca/~bouchard/GEP/README-usage.html
 * 
 * @author Alexandre Bouchard
 * @author Ardavan Saeedi
 */
package gep.util;
import java.util.Random;


public class TP
{
  public static double sample(final Random rand, final double alpha, final double beta)
  {
    if (!(alpha > 0 && beta > 0))
      throw new RuntimeException();
    final double u = rand.nextDouble(); 
    final double u1a = Math.pow(u, 1.0/alpha);
    final double result = beta * (1 - u1a) / u1a;
    if (!(result > 0))
      throw new RuntimeException("TP.sample(alpha=" + alpha + ",beta=" + beta +") returned " + result + "; u=" + u + ",u1a=" + u1a);
    return result;
  }
  public static double mean(double alpha, double beta)
  {
    if (alpha <= 0 || beta <= 0) throw new RuntimeException();
    if (alpha <= 1) return Double.POSITIVE_INFINITY;
    return beta * (alpha / (alpha - 1) - 1);
  }
  
  public static void main(String [] args)
  {
    
    double cur = Double.POSITIVE_INFINITY;
    
    for (int i = 0; i < 10000000; i++)
    {
      double term = (Math.log(i) - Math.log(i+1)) * i;
      if (term > cur)
        throw new RuntimeException(""+i);
      cur = term;
    }
    
    
    double as = 0.0;
    
    for (int i = 0; i < 100000000;i ++)
    {
      as += Math.pow(((double) i)/ ((double) i + 1.0), i);
      if (i % 1000000 == 0)
        System.out.println(as);
    }
    
    
    
    final double alpha = 0.1, beta = 0.1;
    
    double sum = 0.0;
    double N = 1000000;
    Random rand = new Random(2);
    
    double curAlpha = alpha;
    double curBeta = beta;
    
    for (int i = 0; i < 10000; i ++)
    {
      double t = sample(rand, curAlpha, curBeta);
      curAlpha++;
      curBeta += t;
      
      System.out.println("" + i + "\t" + t);
    }
    
    System.out.println("---");
    
    for (int i = 0; i < N; i++)
    {
      double cSample  =sample(rand, alpha, beta);
      System.out.println(cSample);
      sum += cSample;
    }

    System.out.println("Analytic:" + mean(alpha,beta));
    System.out.println("MC:" + (sum/N));
  }
}
